@@ -1,9 +1,17 @@
-c      kcsame:  examine if all array elements are the same
+#if defined NEXT486
+#define IMAG_P dimag
+#elif defined PCLinux
+#define IMAG_P dimag
+#else
+#define IMAG_P imag
+#endif
+      
 c      kinout:  judge if a point is inside of a polygon
 c      kxplbx:  box and line x-point
 c      kxplcy:  cyl and line x-point
 c      kxplp:   plane and line x-point
 c      kxplsl:  line and segment of line x-point
+c      kxplineseg: x-point of two line segments 
 c      kxplPrism: line and prism x-point
 c      k3dclp:  3d clipping
 c      kfige:   find int data pos. >= a given value
@@ -21,6 +29,7 @@ c      kshiftC: shift character until blank  appears
 c      kDebye:  Debye function
 c      kBern:   Bernuoulli number
 c      kalpha:  see if string starts with a-z or A-Z
+c      kxplellip: crossing point of a line with an ellipse
 c      ****************************
       logical function kalpha(string)
 c
@@ -553,7 +562,7 @@ c
 c        see if not right angle crossing
       if( real(z21) .ne. 0.) then
 c           tan of relative angle
-         t=dimag(z21)/real(z21)
+         t=IMAG_P(z21)/real(z21)
       else
          t=1.
       endif
@@ -561,9 +570,9 @@ c          see if they are //
       if( abs(t) .gt. eps )then
 c              non //, so find + point
 c          see if z1 = z2
-          if(dimag(z21) .eq. 0.) then
+          if(IMAG_P(z21) .eq. 0.) then
 c              see if z0-line crosses the z1 and z2
-             if(dimag(z10) .eq. 0.) then
+             if(IMAG_P(z10) .eq. 0.) then
 c                z1 is on the line
                 q=real(z10)
                 p=0.
@@ -574,7 +583,7 @@ c                z1 is on the line
                  icon=2
              endif
          else
-             p=- dimag(z10)/dimag(z21)
+             p=- IMAG_P(z10)/IMAG_P(z21)
              q= real(z10) +  real(z21) * p
              if(p .lt.-eps .or.  p .gt. 1.+eps) then
                  icon=2
@@ -589,23 +598,23 @@ c           if z0=z1  goto 200
              t2=z20*conjg(z20)
              if( t1 .le. t2  ) then
 c                z0 is closer to z1, so examine // of z0-line and z0-z2
-                 t= dimag(z20)/real(z20)
+                 t= IMAG_P(z20)/real(z20)
              else
-                 t=dimag(z10)/real(z10)
+                 t=IMAG_P(z10)/real(z10)
              endif
              if( abs(t) .gt. eps ) then
-                 q=dimag(z10)
+                 q=IMAG_P(z10)
                  icon=3
                  i200=0
              else
                  i200=1
              endif
          else
-             if(abs(dimag(z10)) .le. eps) then
+             if(abs(IMAG_P(z10)) .le. eps) then
                 i200=1
              else
                 i200=0
-                q=dimag(z10)
+                q=IMAG_P(z10)
                 icon=3
              endif
          endif
@@ -1945,9 +1954,167 @@ c                                  if loc2 cannot be found loc2 = 0
       endif
 
       end
+cc     test kxplineseg
+c      real*8 x1, y1,x2,y2, x3, y3, x4, y4, eps
+c      real*8 x,y, p, q
+c      integer icon
+c      eps  = 1.d-12
+c      read(*,*) x1,y1
+c      read(*,*) x2,y2
+c      read(*,*) x3,y3
+c      read(*,*) x4,y4
+c
+c      call kxplineseg(x1, y1, x2, y2, x3, y3, x4, y4, eps,
+c     *     x, y, p, q, icon)
+c      write(*,*) ' icon =', icon, 'p,q=',p,q
+c      write(*,*) ' x,y=',x,y
+c      end
+c
+c
+c     get x-ssing point of two line segments on (x,y) plane
+c
+      subroutine kxplineseg(x1,y1, x2, y2, x3, y3, x4, y4, eps,
+     *     x, y, p, q, icon)
 
+      implicit none
+      real*8 x1, y1, x2, y2  ! input one line segment. in complex ( z1, z2)
+      real*8 x3, y3, x4, y4  !  input the other line segment. in complex  (z3, z4)
+      real*8  eps            !  input. to judge the parallelity and/or
+                             !         overlapping of two line segment
+      real*8  x, y          ! output. obtained x-ssing point
+      real*8  p             ! output. (z1 to the x-ppint)/(z1 to z2).  0<=p<=1. if not  outside
+      real*8  q             ! output. (z3 to the x-point)/(z3 to z4).  0<=q<=1. if not  outside
+      integer icon          ! output.  0, x,y obtained.
+                            !          1, two segment overlap, (x,y) will 
+                            !             be somewhere on the line
+                            !          2, no x-ssing point 
+      complex*16  expia, z1, z2, z3, z4
+      real*8 cosa, sina
+      real*8  length
       
+      z1 = cmplx(x1, y1)
+      z2 = cmplx(x2, y2)
+      z3 = cmplx(x3, y3)
+      z4 = cmplx(x4, y4)
+      length = abs(z4-z3)
+      if( length .le. eps )  then
+         icon = 2
+      else
+         cosa = (x4-x3)/length
+         sina = (y4-y3)/length
+         expia = cmplx(cosa, sina)
+         call kxplsl(z3, expia, z1 ,z2, eps, p, q, icon)
+         if(icon  .eq. 0) then
+c             see if x-point is within z3-z4.
+            if(q .lt. 0.  .or. q .gt. length ) then
+c                point is outside of the segment
+               icon = 2
+            else
+               q = q/length
+            endif
+         elseif(icon .eq. 1 ) then
+c///////
+c            write(0,*) ' p,p=',p,q
+            if( (x3-x1)*(x2-x3) .gt. 0.) then
+               x = x3
+               y = y3
+            elseif( (x1-x3)*(x4-x1) .gt. 0.) then 
+               if( (x2-x3)*(x4-x2) .gt. 0.) then 
+                  if(abs(x1-x3) .lt. abs(x2-x3) ) then
+                     x = x1
+                     y = y1
+                  else
+                     x = x2
+                     y = y2
+                  endif
+               else
+                  x = x1
+                  y = y1
+               endif
+            elseif( (x2-x3)*(x4-x2) .gt. 0.) then 
+               x = x2
+               y = y2
+            else
+               icon = 2 
+            endif
+         endif
+      endif
+      if(icon .eq. 0) then
+         x = x1 +p*(x2-x1)
+         y = y1 +p*(y2-y1)
+      endif 
+      end
+c           test
+c      implicit none
+c      real*8 a, b, x0, y0, dirx, diry, x1, x2, y1, y2,l1,l2
+c      integer cross
+c      a = 2.
+c      b = 1.
+c      x0 = 0.5
+c      y0 = -3.0
+c      dirx = 0.1d0
+c      diry = sqrt(1.d0-dirx**2)
+c      call kxplellip(a, b, x0, y0, dirx, diry, l1, l2, cross)
+c      write(0,*) ' cross=',cross, x0+l1*dirx,  y0+l1*diry,
+c     *          x0+l2*dirx, y0+l2*diry
+c      end
+c
+c
+      subroutine kxplellip(a, b, x0, y0, dirx, diry, l1, l2, cross)
+c         get crossing point of a line with an ellipse whose center is at (0,0)
+c
+      implicit none
+      real*8 a  ! input   x radis of the ellipse
+      real*8 b  ! input   y //
+      real*8 x0 ! input   the line start from this x
+      real*8 y0 ! input   the line start from this y
+      real*8 dirx  ! input the line's direction cos to x
+      real*8 diry  ! input the line's //               y
+      real*8 l1    !  output.  a cross point ; x0+l1*dirx, y0+l1*diry
+      real*8 l2    !  output.  the other cross point
+                   !      0 <l1<l2 or l2 <0<l1 or l2<l1<0. or l1=l2
+      integer cross ! output.  0--> the line cross the ellipse at two point
+                    !          1--> the line is tangnetial to the ellipse
+                    !          if two point are too near,  we regards them as one.
+                    !          -1--> the line dose not cross the ellipse
+      real*8   x, y, wx, wy, c0, c1h, c2, dq
+      real*8   small, temp
+      data small/1.d-8/
       
+
+      x = x0/a
+      y = y0/b
+      wx = dirx/a
+      wy = diry/b
+      c2= wx**2 + wy**2
+      c1h = x*wx+ y*wy
+      c0 = x**2 + y**2 - 1.d0
+
+      dq = c1h**2 - c0*c2
       
-      
-      
+      if( dq .lt. 0.)  then
+         cross = -1
+      elseif( dq .le. small) then
+         cross = 1
+      else
+         cross = 0
+      endif
+      if( cross .ge. 0 ) then
+         dq =sqrt(dq)
+         l1 = (-c1h - dq)/c2
+         l2 = (-c1h + dq)/c2
+         if(l1*l2 .lt. 0.) then
+            if(l1 .lt. 0.) then
+               temp = l1
+               l1 = l2
+               l2 = temp
+            endif
+         else
+            if(abs(l1) .gt. abs(l2)) then
+               temp = l1
+               l1 = l2
+               l2 = temp
+            endif
+         endif
+      endif
+      end
