@@ -55,13 +55,14 @@ c
         include 'Zother.h'
         character*150 msg
 c
+        write(*,*) '------------------------------------------------'
         open(11, file='./other')
         read(11,*) MulLow, energy, xin, yin, zin, outatint
         close(11)
-        write(msg, *) ' MulLow=',MulLow, ' energy=',energy,
+        writse(0, *) ' MulLow=',MulLow, ' energy=',energy,
      *      ' xin, yin,zin=',xin, yin, zin, ' outatint=',
      *        outatint
-        call cerrorMsg(msg, 1)
+c        call cerrorMsg(msg, 1)
         if(outatint) then
            write(*,*)
      *     '#  "eta"  "y" "pt" " 1/pt" "E/E0(lab)" " code"'
@@ -145,7 +146,7 @@ c       *  If default is ok, don't touch this.
               write(*,*) 
               write(*,'(5g14.4,i3)')
      *             sngl(eta), sngl(y),sngl(pt),sngl(pti),
-     *             sngl(po.fm.p(4)/plab.fm.p(4)), w(i).code,
+     *             sngl(po.fm.p(4)/plab.fm.p(4)), w(i).code
            else
               aTrack.p = w(i)
               aTrack.pos.x = xin
@@ -372,7 +373,7 @@ c       *  for the event.
 c       *
         subroutine ue1ev
         implicit none
-        implicit none
+
 
         write(*,*) 0,0,0,0,0,0,0,0,0,0,0,0,0
         end
