@@ -1,4 +1,6 @@
 #include "ZsubstRec.h"
+c        epifCross is changed radically.
+c        epbndry; icon is changed to get el in real*8
 c        *********************************************************
 c        *
 c        *  epgen:generate  showers until stack area becomes empty
@@ -142,7 +144,7 @@ c                 icon=1-->no user routine for incident
           write(filen, *) TraceDir(1:klena(TraceDir))//'/trace',
      *         Nevrun + 1
           call kseblk(filen, ' ', leng)
-          call copenfw(IoTrace, filen,  jcon)
+          call copenfw(abs(IoTrace), filen,  jcon)
           if(jcon .ne. 0) then
              call cerrorMsg('**************** Fatal error ', 1)
              call cerrorMsg(
@@ -169,7 +171,7 @@ c      ******************* end of 1 event
 c               user dependent end process
        if(Move.Abort .le. 1) call ue1ev
        if(Trace) then
-          close(IoTrace)
+          close(abs(IoTrace))
        endif
        end
 c      *************  now 1 event is ready to start
@@ -230,187 +232,7 @@ c            inquire  current event number created in this run
        integer nev
        nev = Nevrun
        end
-      subroutine eppush(aTrack)
-      implicit none
-#include  "ZepTrack.h"
-#include  "ZepStack.h"
 
-        record /epTrack/ aTrack
-        character*80 msg
-
-        integer i
-       
-        Stack_pos = Stack_pos + 1
-
-        if(Stack_pos .gt. Max_stack_size) then
-           write(msg, *) 'stack overflow; max=',Max_stack_size
-           call cerrorMsg(msg, 1)
-           call cerrorMsg(
-     *     'You may enlarge EPMAX_STACK in ZepMaxdef.h',1)
-           call cerrorMsg(
-     *     'the last 100  stacked tracks are listed on stdout',1)
-c
-           write(*, *)
-     *         ' #  k charge    e            x           z         wz'
-           do i = Stack_pos, Stack_pos - 100, -1
-              write(*, *) i, Stack(i).p.code, 
-     *             Stack(i).p.charge, Stack(i).p.fm.p(4),
-     *             Stack(i).pos.x,
-     *             Stack(i).pos.y,
-     *             Stack(i).w.z
-           enddo
-           stop 9999
-        endif
-        Stack(Stack_pos) = aTrack
-
-        end
-c       ************************
-c                 put 1ptcl in the stack area (for unknown cn)
-        subroutine epputTrack(aTrack)
-       implicit none
-#include "ZepTrack.h"
-#include "Zep3Vec.h"
-#include "Zcnfig.h"
-
-        record /epTrack/ aTrack  ! input. 
-
-c                 find component # for pos (world coord.)
-c              and  set it in aTrack.cn
-        call eppos2cn(0,  aTrack,  aTrack.cn)
-
-c         convert pos/w into  l.c
-        if(aTrack.cn .le. Det.nct ) then
-
-           call epw2l(aTrack.cn, aTrack.pos, aTrack.pos)
-           call epw2ld(aTrack.cn, aTrack.w,  aTrack.w)
-           call eppush(aTrack)
-        else
-           call
-     *     cerrorMsg(' epputTrack: ptcl is not in detector', 0)
-        endif
-      end
-c       ************************
-c         put 1 ptcl in the stack disk (for unknown cn)
-c   
-        subroutine epputTrk2(io, aTrack)
-       implicit none
-#include "ZepTrack.h"
-#include "Zep3Vec.h"
-#include "Zcnfig.h"
-#include "ZepStack.h"
-
-       integer io  !  disk file number to stack the track
-       record /epTrack/ aTrack  ! input. 
-
-c                 find component # for pos (world coord.)
-c              and  set it in aTrack.cn
-        call eppos2cn(0,  aTrack,  aTrack.cn)
-
-c         convert pos/w into  l.c
-        if(aTrack.cn .le. Det.nct ) then
-           call epw2l(aTrack.cn, aTrack.pos, aTrack.pos)
-           call epw2ld(aTrack.cn, aTrack.w,  aTrack.w)
-           write(io) aTrack
-           StackDisk = io
-        else
-           call
-     *     cerrorMsg(' epputTrk2: ptcl is not in detector', 0)
-        endif
-      end
-c     ********************
-      subroutine epnewComp
-      implicit none
-#include "ZepTrackv.h"
-#include "Zcnfig.h"
-c         when a particle moves  to a new component
-c         the following must be done
-      Cn = cTrack.cn
-      MediaNo = Det.Cn2media(Cn)
-
-c       r.l                         cm   / X0           
-      MaxPath =abs( Det.cmp(Cn).MaxPathL/Media(MediaNo).X0)
-c         Det.cmp(Cn).MaxPathL may be < 0; which means
-c        that the value is set by Epics and abs should be used.
-c         already dummy
-      call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
-
-
-      end
-
-
-c      ******************* 
-       subroutine epempty
-       implicit none
-c             clear the stack
-#include  "ZepTrack.h"
-#include  "ZepStack.h"
-
-           Stack_pos = 0
-      end
-c     ******************************
-      subroutine eppop(aTrack, icon)
-      implicit none
-c             get 1 particle from the stack area
-c              icon=0:  1 ptcl gotten
-c                  -1:  no more ptcl
-#include  "ZepTrackv.h"
-#include  "ZepStack.h"
-
-       
-c
-       record /epTrack/ aTrack
-       integer icon
- 10    continue
-c         do 876 until (ok)
-       if(Stack_pos .gt. 0) then
-          icon=0
-c                vectors are in local.c
-          aTrack = Stack(Stack_pos)     
-c                make cosmos ptcl; e may not be informed. g may  not be,
-c                                  if IncGp=0.
-          call ep2cosPtcl(aTrack.p)
-
-          Stack_pos = Stack_pos - 1
-          Cn = aTrack.cn   ! this may be used in epchckE
-c              do business when a new component appear
-          call epnewComp
-          call epchckE(aTrack, icon)
-          if(icon .ne. 0) goto 10
-
-       else
-          if(StackDisk .gt. 0) then
-             read(StackDisk, end=100) aTrack
-             call eppush(aTrack)
-             goto 10
-          endif
- 100      continue
-          StackDisk = 0
-          icon=-1
-       endif
- 876    continue
-
-        end
-c       *************************
-        subroutine epgetTrack(n, aTrack, icon)
-        implicit none
-c            returns a specified track in the stack
-c                 
-#include  "ZepTrack.h"
-#include  "ZepStack.h"
-#include  "Zcode.h"
-c
-        integer n  ! input. track in n-th stack position
-        record /epTrack/ aTrack  ! output
-        integer icon             ! output. 0--> ok
-                                 !        non 0--> no n-th stack
-        if(n .ge. 1 .and. n .le. Stack_pos) then                   
-           aTrack = Stack(n)
-           icon = 0
-        else
-           icon = 1
-        endif
-       end
-c       #####################################
        subroutine epfl1
        implicit none
 c           follow 1 particle
@@ -418,6 +240,7 @@ c              until current partile dies or all are put into stack.
 #include  "ZepTrackp.h"
 #include  "ZepTrackv.h"
 #include  "Zcnfig.h"
+#include  "ZepManager.h"
 
       integer icon, info
 
@@ -456,6 +279,9 @@ c           adjust momentum; because of  energy change
             endif
 
 c            if(.not. Move.Cross .and. cTrack.p.charge .ne. 0) then
+c               we don't use above judgement; later at epCross, we use new
+c               angle due to scattering at Move.boundary. (don't use
+c               new  position due to scattering in epCross)
 c           ++++++++++
             if(cTrack.p.charge .ne. 0) then
 c                 multiple scattering, magnetic deflection
@@ -465,7 +291,10 @@ c                 electric deflection
             endif
 c              take trace info.
             if(Trace) then
-               call epTrace
+               if(IoTrace .lt. 0 .or. (IoTrace .gt. 0 
+     *           .and. cTrack.p.charge .ne. 0)) then
+                  call epTrace
+               endif
             endif
 c              really move
 #ifdef  SUBSTREC
@@ -482,7 +311,10 @@ c                       icon =  1 always.
          else
 c              take trace info. of dead particle.
             if(Trace) then
-               call epTrace
+               if(IoTrace .lt. 0 .or. (IoTrace .gt. 0 
+     *           .and. cTrack.p.charge .ne. 0)) then
+                  call epTrace
+               endif
             endif
 
          endif
@@ -554,7 +386,9 @@ c              local. coord..
 c     *************** see if Move.Track crosses the boundary ******
 c              set Cross=t/f, if t, adjust path and set Trunc=t
       call epifCross(icon)
-      if(icon .eq. 0) then
+      if(icon .eq. 2) return  !  ptcl went void by delicate boundary.
+      icon = 0
+cc      if(icon .eq. 0) then
 c     
 c     ************** energy loss consideration ******************
 c        if E becomes <= m,  adjust path and set Trunc=t,
@@ -564,7 +398,7 @@ c        and reset Cross.
          else
             Move.dE = 0.
          endif
-      endif
+cc      endif
       end
 c     *******************************
       subroutine epCross(icon)
@@ -585,10 +419,10 @@ c                                 other values are not given here
 c
 c      save the cTrack
       aTrack = cTrack
-c       cross the boundary
+c       cross the boundary;  use  new angle at the boundary.
+c      (not cTrack.w.x etc)
       cTrack.pos.x = Move.boundary.x +
-c     *     EpsLeng* Move.Track.w.x
-     *     EpsLeng* cTrack.w.x   !+++++++++
+     *     EpsLeng* Move.Track.w.x  
 c          if EpsLeng and w.x are both small, and x is large
 c          no move may happen, avoid such case.
 c               You may give larger EpsLeng in input data.
@@ -598,33 +432,31 @@ c
          do while( cTrack.pos.x .eq. Move.boundary.x .and.
      *             n .lt. 10 ) 
             cTrack.pos.x = Move.boundary.x +
-     *           n*EpsLeng* cTrack.w.x   
+     *           n*EpsLeng* Move.Track.w.x
             n = n * 2
          enddo
       endif
 
       cTrack.pos.y = Move.boundary.y +
-c     *     EpsLeng* Move.Track.w.y 
-     *     EpsLeng* cTrack.w.y   !+++++++
+     *     EpsLeng* Move.Track.w.y
       if( cTrack.w.y .ne. 0.) then
          n = 2
          do while( cTrack.pos.y .eq. Move.boundary.y
      *             .and. n .lt. 10 ) 
             cTrack.pos.y = Move.boundary.y +
-     *           n*EpsLeng* cTrack.w.y   
+     *           n*EpsLeng*  Move.Track.w.y
             n = n * 2
          enddo
       endif
 
       cTrack.pos.z = Move.boundary.z +
-c     *     EpsLeng* Move.Track.w.z
-     *     EpsLeng* cTrack.w.z   !++++++++++
+     *     EpsLeng* Move.Track.w.z
       if(cTrack.w.z .ne. 0. ) then
          n = 2
          do while( cTrack.pos.z .eq. Move.boundary.z  .and.
      *          n .lt. 10 )
             cTrack.pos.z = Move.boundary.z +
-     *           n*EpsLeng* cTrack.w.z
+     *           n*EpsLeng* Move.Track.w.z
             n = n * 2
          enddo
       endif
@@ -635,15 +467,19 @@ c     *     EpsLeng* Move.Track.w.z
       call epsubvec(Move.Track.w, cTrack.w)
 #endif
 
-c         new comp. number
+c         new comp. number;  new position would be, in principle,
+c         in a new component. but scattering at the boundary may
+c         cause the new pos to be in the same component.
       call eppos2cn(Cn, cTrack, cnx)
-      if(cnx .gt. Det.nct) then
+      if(Cn .eq. cnx) then
+         Move.Cross = .false.
+      elseif(cnx .gt. Det.nct) then
          info = 0   ! exiting to void
       else
          info = -cnx   ! exiting to cnx
       endif
       cTrack.cn = cnx
-      if(Det.cmp(Cn).CountIO .ge. 2) then
+      if(Det.cmp(Cn).CountIO .ge. 2 .and. Move.Cross) then
 c          user hook for  counting exiting ptcls 
 c                    exiting from Cn to cnx
          call userbd(info, aTrack, Move, Media(MediaNo))
@@ -663,7 +499,7 @@ c////////////////
       if(cnx .gt. Det.nct) then
 c          void
          icon = 1
-      else
+      elseif(Move.Cross) then
          info = Cn            ! save the current cn
 c         update coord. to local one in new  comp.
          call epl2w(Cn, cTrack.pos, postemp)
@@ -685,7 +521,7 @@ c            update comp. info (Cn, MediaNo etc are updated)
 
          if(Det.cmp(Cn).CountIO .eq. 1 .or.
      *      Det.cmp(Cn).CountIO .eq. 3 ) then
-c             count entering ptcls;  from Cn to Move.Track.cn
+c             count entering ptcls;  from info to Move.Track.cn (=Cn)
             call userbd(info, aTrack, Move, Media(MediaNo)) 
             if(Move.Abort .ne. 0) then
                if(Move.Abort .eq. 3) then
@@ -921,6 +757,7 @@ c     ******************
 c     ******************
 c         take trace of paritcles
        implicit none
+#include  "ZepTrackp.h"
 #include  "ZepTrackv.h"
 #include  "Zcode.h"
 #include  "ZepManager.h"
@@ -928,10 +765,10 @@ c         take trace of paritcles
 c          standard Trace information
       record /epPos/ posbw, posw
       real*8 xxx/-1.d37/, yyy/-1.d36/, zzz/1.d34/ 
-      integer kkk/-1000/
+      integer kkk/-1000/, chkchg/-1000/
 
       integer k
-      save xxx, yyy, zzz, kkk
+      save xxx, yyy, zzz, kkk, chkchg
 c
 c          Trace informaion  ccc if for putting only electorns
 c
@@ -942,28 +779,46 @@ c             convert to world coord. starting point of the segment
 ccc      if(k .eq. kelec) then
          if(kkk .ne. k .or. xxx .ne. posbw.x .or. 
      *                      yyy .ne. posbw.y .or.
-     *                      zzz .ne. posbw.z) then
+     *                      zzz .ne. posbw.z .or. 
+     *                   chkchg .ne. cTrack.p.charge ) then
             if(xxx .ne. -1.d37) then
-               write(IoTrace, *)
-               write(IoTrace, *)
+               write(abs(IoTrace), *)
+               write(abs(IoTrace), *)
+            endif
+            if(TimeStruc) then
+               write(abs(IoTrace),
+     *          '(3g16.8, i4, g11.4, i4, g16.8)')
+     *            posbw.x, posbw.y, posbw.z, k,
+     *            Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *            cTrack.p.charge,
+     *            cTrack.t
+            else
+               write(abs(IoTrace),'(3g16.8,i4,g11.4, i4)')
+     *            posbw.x, posbw.y, posbw.z, k,
+     *            Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *            cTrack.p.charge
             endif
-            write(IoTrace, *) 
-     *        sngl(posbw.x), sngl(posbw.y), sngl(posbw.z), k,
-     *        sngl(cTrack.p.fm.p(4)), cTrack.p.charge
-c     *       , Nevrun+1
-ccc         write(IoTrace, *)  sngl(posbw.x), sngl(posbw.z)
          endif
 c            end point of  the segment
          call epl2w(Cn, Move.Track.pos, posw)
-         write(IoTrace, *) 
-     *       sngl(posw.x), sngl(posw.y), sngl(posw.z), k, 
-     *       sngl(Move.Track.p.fm.p(4)), Move.Track.p.charge
-c     *     ,  Nevrun+1
-ccc      write(IoTrace, *) sngl(posw.x), sngl(posw.z)
+         if(TimeStruc) then
+            write(abs(IoTrace),'(3g16.8,i4,g11.4, i4, 
+     *         g16.8)')
+     *           posw.x, posw.y, posw.z, k,
+     *           Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *           Move.Track.p.charge,
+     *           Move.Track.t
+         else
+            write(abs(IoTrace),'(3g16.8,i4,g11.4, i4)')
+     *           posw.x, posw.y, posw.z, k,
+     *           Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *           Move.Track.p.charge
+         endif
          xxx = posw.x
          yyy = posw.y
          zzz = posw.z
          kkk = k
+         chkchg = Move.Track.p.charge
 ccc      endif
        end
 c      ****************************************
@@ -1181,138 +1036,43 @@ c             see if Move.Track crosses a boundary and set Cross = t/f
 c        If t, reset Move.Track.pos just before the boundary.
 c        set exact boundary in Move.boundary.
 c
+      integer kcon, icon
 
 
-      integer icon, cnxx, i, cond, kcon
-      logical bfound, sameComp
-      real*8 el, tempel
+      real*8 el
 
 c         see if Move.Track is in Cn-th component.
 c          coord is local coord.
-
-      call epsOne(Cn, Move.Track.pos, icon)
-
-
-      if(icon .eq. 0) then
-c          Move.Track is in the same component. But it may 
-c          entered into an inner Matreshka; examine it.
-         call epsMatreska(Cn, Move.Track.pos, cnxx)
-c ////////////////
-c         call debugpos('ifCross; matreska',Cn, 
-c     *       Move.Track.pos, cnxx)
-c///////////////
-         if(cnxx .ne. Cn) then
-c              child matreshka found, so that
-c              Move.Track is outside of the current volume
-            icon = 1
+      call epbndry(Move.boundary, el, icon)
+c       icon =1 indicates something wrong. but 
+c       it should have been corrected safely so we don't
+c       care.
+c       icon =2  : fail to find boundary. due to delicate problem
+      if(icon .ne.  2) then
+         if(Move.dl .lt. el) then
+            kcon = 0
+            Move.Cross = .false.
          else
-c            then examine if the pos in some union part.
-
-            call epsPCont(Cn, Move.Track.pos, icon)
-c     /////////////////
-c            call debugpos('ifCross; epsPCont', Cn, 
-c     *          Move.Track.pos, icon)
-c /////////////
-c             icon !=0 means, pos is outside of the union
-c             part of the current volume already
+            kcon = 1
+            Move.Cross = .true.
+            Move.Trunc = .true.
+            Move.Track.pos.x = Move.boundary.x -
+     *           EpsLeng* cTrack.w.x
+            Move.Track.pos.y = Move.boundary.y -
+     *           EpsLeng* cTrack.w.y
+            Move.Track.pos.z = Move.boundary.z -
+     *           EpsLeng* cTrack.w.z
+c            Move.dl = sqrt(
+c     *          (Move.Track.pos.x-cTrack.pos.x)**2 +
+c     *          (Move.Track.pos.y-cTrack.pos.y)**2 +
+c     *          (Move.Track.pos.z-cTrack.pos.z)**2 )
+            Move.dl = el - EpsLeng
+            Move.dt = Move.dl/Media(MediaNo).X0
+            Move.dx = Move.dl/Media(MediaNo).gtocm 
          endif
-      endif
-c                judge if in the same component
-      bfound = .false.  ! boundary is not yet found
-      sameComp = icon .eq. 0
-
-      
-      if(sameComp)  then
-c         Move.Track is in Cn-th comp. but  may
-c         crosses an inner matreska. examine it.
-c         e.g; starting point S and ending point E
-c         are in the same component but it crosses
-c         the inner matreshka; 
-c                  
-c                  S   
-c                   \
-c                    \   * *
-c                      *  Inner Matreshka
-c                     *\       *
-c                     * \      *
-c                      * \    *
-c                        *\* *
-c                          \
-c                           E       
-
-         if(Det.cmp(Cn).NMatreska .gt. 0) then
-c            current comp.  is Matreska  so that
-c            we have to examine inner  comp.
-c            length to the new position.
-            el = sqrt((Move.Track.pos.x-cTrack.pos.x)**2 +
-     *                (Move.Track.pos.y-cTrack.pos.y)**2 +
-     *                (Move.Track.pos.z-cTrack.pos.z)**2 )
-            do i = 1, Det.cmp(Cn).NMatreska
-c               length to the boundary of the Det.cmp(Cn).Contains(i)-th
-c               component which is i-th matreshka of Cn-th comp.
-c(((((((((((
-c               call epbndry2(Det.cmp(Cn).Contains(i), tempel, icon)
-               call epbndry2(CnArea(Det.cmp(Cn).Contains+i),
-     *          tempel, icon)
-c))))))))))))))
-c          //////////
-c               call debugpos('ifCross; epbndry2',Cn, 
-c     *          Move.Track.pos, icon)
-c//////////////////
-c                  icon = 1; pos is   outside of the examined comp.
-c                    and the length is tempel.
-               if(icon .eq. 1 .and. tempel .lt. el) then
-c                    really crosses
-                  el = tempel
-                  bfound = .true.
-                  sameComp = .false.
-               endif
-            enddo
-         endif
-      endif
-      if(sameComp) then
-         Move.Cross =  .false.
       else
-         Move.Cross = .true.
-         Move.Trunc = .true.
-c          current component to another one; get boundary
-         if(.not. bfound) then
-c                    we have to find boundary; it may be
-c                the one  with some matreska inside.
-            call epbndry(Move.boundary, cond)
-c////////////////////
-c            call debugpos('ifCross; epbndry', Cn, 
-c     *       Move.boundary, cond)
-c///////////////
-            if(cond .eq. 2) then
-               call cerrorMsg('boundary not found; strange',0) 
-            elseif(cond .eq. 1) then
-               kcon = 1
-               return  !  emergency
-            endif
-         else
-            Move.boundary.x = cTrack.pos.x + el* cTrack.w.x
-            Move.boundary.y = cTrack.pos.y + el* cTrack.w.y
-            Move.boundary.z = cTrack.pos.z + el* cTrack.w.z
-         endif 
-         Move.Track.pos.x = Move.boundary.x -
-c     *             EpsLeng* Move.Track.w.x
-     *             EpsLeng* cTrack.w.x
-         Move.Track.pos.y = Move.boundary.y -
-c     *             EpsLeng* Move.Track.w.y
-     *             EpsLeng* cTrack.w.y
-         Move.Track.pos.z = Move.boundary.z -
-c     *             EpsLeng* Move.Track.w.z
-     *             EpsLeng* cTrack.w.z
-         Move.dl = sqrt(
-     *          (Move.Track.pos.x-cTrack.pos.x)**2 +
-     *          (Move.Track.pos.y-cTrack.pos.y)**2 +
-     *          (Move.Track.pos.z-cTrack.pos.z)**2 )
-
-         Move.dt = Move.dl/Media(MediaNo).X0
-         Move.dx = Move.dl/Media(MediaNo).gtocm 
+         kcon = 2
       endif
-      kcon  = 0
       end
 c
       subroutine epchckE(aTrack, icon)
@@ -1465,22 +1225,34 @@ c      *****************
 c        new position is assumed to be fixed.
 c        (however, before scattering, and energy check).
 c        update time and take trace
+#include  "Zglobalc.h"
 #include  "ZepTrackp.h"
 #include  "ZepTrackv.h"
 
-       real*8 betai    !  1/beta
+       real*8 ctau, u
+       real*8 beta1, betaav
 c
        if(Move.Track.p.mass .eq. 0.) then
-          betai=1.d0
-       elseif(Move.Track.p.fm.p(4)  .gt. Move.Track.p.mass*1000.)then
-          betai=1. + (Move.Track.p.mass /Move.Track.p.fm.p(4))**2/2
-       elseif(Move.Track.p.fm.p(4) .gt. Move.Track.p.mass) then
-          betai=1./
-     *         sqrt(1.d0 - (Move.Track.p.mass/Move.Track.p.fm.p(4))**2)
+          betaav=1.d0
        else
-          betai=1.d8
+          call cgetBeta( Move.Track.p,  betaav)
+          if(betaav .lt. 0.98) then
+             call cgetBeta(cTrack.p, beta1)
+             betaav = (betaav+beta1)/2.
+          endif
+       endif
+       if(betaav .gt. 0.) then
+          Move.Track.t = cTrack.t + Move.dl/betaav
+       else
+c               stopped one. add decay time if possible
+          call cgetctau(cTrack.p, ctau)
+          if(ctau .eq. Infty) then
+             Move.Track.t= cTrack.t+1.e8
+          else
+             call rndc(u)
+             Move.Track.t = cTrack.t - log(u)*ctau
+          endif
        endif
-       Move.Track.t = cTrack.t + Move.dl * betai
        end
 
 c      ************************
